package net.plil.clubinfo.etunicorn.app.consommation;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;

import net.plil.clubinfo.etunicorn.R;
import net.plil.clubinfo.etunicorn.app.MainActivity;
import net.plil.clubinfo.etunicorn.data.Consommation;
import net.plil.clubinfo.etunicorn.utils.VolleyUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;


public class ModifyConsommation extends DialogFragment {

    EditText mNomConsomation;
    EditText mPrice;
    TextInputLayout mPriceInput;
    TextInputLayout mNomInput;
    ProgressBar mProgressBar;

    public static ModifyConsommation newInstance(Consommation consommation) {
        ModifyConsommation f = new ModifyConsommation();

        // Supply num input as an argument.
        Bundle args = new Bundle();
        args.putSerializable("consommation", consommation);
        f.setArguments(args);

        return f;
    }


    @Override
    @NonNull
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        // Use the Builder class for convenient dialog construction
        final LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.fragment_modify_consommation, null);
        mNomConsomation = (EditText) view.findViewById(R.id.modify_consommation_name);
        mPrice = (EditText) view.findViewById(R.id.modify_consommation_price);
        mProgressBar = (ProgressBar) view.findViewById(R.id.modify_consommation_progress_bar);
        mNomInput = (TextInputLayout) view.findViewById(R.id.modify_consommation_input_name);
        mPriceInput = (TextInputLayout) view.findViewById(R.id.modify_consommation_input_price);

        final Consommation consommation = (Consommation) getArguments().getSerializable("consommation");
        assert consommation != null;

        mNomConsomation.setText(consommation.getNomConsomation());
        mPrice.setText(String.format(Locale.US, "%.2f", consommation.getPrix()));

        AlertDialog dialog = new AlertDialog.Builder(getActivity())
                .setTitle(R.string.modify)
                .setView(view)
                .setPositiveButton(R.string.modify, null)
                .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyConsommation.class);
                    }
                }).create();
        // Create the AlertDialog object and return it

        dialog.setOnShowListener(new DialogInterface.OnShowListener() {
            @Override
            public void onShow(DialogInterface dialog) {
                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        boolean inputOk = true;
                        if (mPrice.getText().toString().isEmpty()) {
                            inputOk = false;
                            mPriceInput.setError(getString(R.string.error_modify_consommation_input_price));
                        } else {
                            mPriceInput.setError(null);
                        }
                        if (mNomConsomation.getText().toString().isEmpty()) {
                            inputOk = false;
                            mNomInput.setError(getString(R.string.error_modify_consommation_input_nom));
                        } else {
                            mNomInput.setError(null);
                        }
                        if (inputOk) {
                            mProgressBar.setVisibility(View.VISIBLE);
                            mPriceInput.setVisibility(View.GONE);
                            mNomInput.setVisibility(View.GONE);
                            JSONObject jsonObject = new JSONObject();
                            try {
                                jsonObject.put("nom", mNomConsomation.getText().toString());
                                jsonObject.put("prix", Double.parseDouble(mPrice.getText().toString()));
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                            JsonObjectRequest jsonObjectRequest = new JsonObjectRequest(Request.Method.PUT, VolleyUtils.baseUri + "/consomation/" + consommation.getIdConsomation(), jsonObject, new Response.Listener<JSONObject>() {
                                @Override
                                public void onResponse(JSONObject response) {
                                    mProgressBar.setVisibility(View.GONE);
                                    Toast.makeText(getContext(), R.string.modify_done, Toast.LENGTH_LONG).show();
                                    dismiss();

                                }
                            }, new Response.ErrorListener() {
                                @Override
                                public void onErrorResponse(VolleyError error) {
                                    mProgressBar.setVisibility(View.GONE);
                                    Toast.makeText(getContext(), R.string.modify_refused, Toast.LENGTH_LONG).show();
                                }
                            }
                            ) {
                                @Override
                                public Map<String, String> getHeaders() throws AuthFailureError {
                                    Map<String, String> headers = new HashMap<>();
                                    headers.put("Authorization", MainActivity.session.getToken());
                                    return headers;
                                }
                            };
                            jsonObjectRequest.setTag(ModifyConsommation.class);
                            VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonObjectRequest);
                        }
                    }
                });
            }
        });
        dialog.setCanceledOnTouchOutside(false);
        return dialog;
    }

    @Override
    public void onStop() {
        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyConsommation.class);
        dismissAllowingStateLoss();
        super.onStop();
    }

}
