package net.plil.clubinfo.etunicorn.app.personne;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;
import com.android.volley.toolbox.JsonObjectRequest;

import net.plil.clubinfo.etunicorn.R;
import net.plil.clubinfo.etunicorn.app.MainActivity;
import net.plil.clubinfo.etunicorn.app.NFCSupport;
import net.plil.clubinfo.etunicorn.data.Personne;
import net.plil.clubinfo.etunicorn.data.Role;
import net.plil.clubinfo.etunicorn.utils.ConvertBytesToString;
import net.plil.clubinfo.etunicorn.utils.JsonConverter;
import net.plil.clubinfo.etunicorn.utils.VolleyUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;


public class ModifyPersonne extends DialogFragment implements NFCSupport {

    Calendar myCalendar = Calendar.getInstance();

    EditText mCarte;
    Spinner mNaissance;
    EditText mLogin;
    Spinner mRole;
    ProgressBar mProgressBar;

    TextInputLayout mCarteInput;
    TextInputLayout mLoginInput;

    private String[] arraySpinnerNaissance;
    private ArrayAdapter<String> adapterNaissance;
    DatePickerDialog.OnDateSetListener date = new DatePickerDialog.OnDateSetListener() {

        @Override
        public void onDateSet(DatePicker view, int year, int monthOfYear,
                              int dayOfMonth) {
            myCalendar.set(Calendar.YEAR, year);
            myCalendar.set(Calendar.MONTH, monthOfYear);
            myCalendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
            updateLabel();
        }
    };
    private List<Role> arraySpinnerRole;
    private ArrayAdapter<Role> adapterRole;

    public static ModifyPersonne newInstance(Personne personne) {
        ModifyPersonne f = new ModifyPersonne();

        // Supply num input as an argument.
        Bundle args = new Bundle();
        args.putSerializable("personne", personne);
        f.setArguments(args);

        return f;
    }

    @Override
    @NonNull
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        myCalendar.set(Calendar.YEAR, myCalendar.get(Calendar.YEAR) - 18);
        // Use the Builder class for convenient dialog construction
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.fragment_modify_personne, null);
        mLogin = (EditText) view.findViewById(R.id.modify_personne_login);
        mCarte = (EditText) view.findViewById(R.id.modify_personne_carte);
        mNaissance = (Spinner) view.findViewById(R.id.modify_personne_naissance);
        mRole = (Spinner) view.findViewById(R.id.modify_personne_role);
        mProgressBar = (ProgressBar) view.findViewById(R.id.modify_personne_progress_bar);

        mLoginInput = (TextInputLayout) view.findViewById(R.id.modify_personne_login_input);
        mCarteInput = (TextInputLayout) view.findViewById(R.id.modify_personne_carte_input);

        final Personne personne = (Personne) getArguments().getSerializable("personne");
        assert personne != null;

        mCarte.setText(personne.getCarte());
        mLogin.setText(personne.getLogin());

        arraySpinnerRole = new ArrayList<>();
        arraySpinnerRole.add(personne.getRole());
        adapterRole = new ArrayAdapter<>(getActivity(),
                R.layout.simple_item_layout, arraySpinnerRole);
        mRole.setAdapter(adapterRole);
        changeRolePossibility();
        mNaissance = (Spinner) view.findViewById(R.id.modify_personne_naissance);
        final String myFormat = "yyyy-MM-dd"; //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, Locale.FRANCE);
        if (personne.getNaissance() == null) {
            arraySpinnerNaissance = new String[]{
                    getString(R.string.modify_personne_hint_date)
            };
        } else {
            arraySpinnerNaissance = new String[]{
                    sdf.format(personne.getNaissance())
            };
        }
        adapterNaissance = new ArrayAdapter<>(getActivity(),
                R.layout.simple_item_layout, arraySpinnerNaissance);
        mNaissance.setAdapter(adapterNaissance);


        AlertDialog dialog = new AlertDialog.Builder(getActivity())
                .setTitle(R.string.modify)
                .setView(view)
                .setPositiveButton(R.string.modify, null)
                .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyPersonne.class);
                    }
                }).create();
        // Create the AlertDialog object and return it
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {
            @Override
            public void onShow(DialogInterface dialog) {
                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        mProgressBar.setVisibility(View.VISIBLE);
                        mRole.setVisibility(View.GONE);
                        mLoginInput.setVisibility(View.GONE);
                        mCarteInput.setVisibility(View.GONE);
                        mNaissance.setVisibility(View.GONE);
                        JSONObject jsonObject = new JSONObject();
                        try {
                            jsonObject.put("carte", mCarte.getText().toString());
                            jsonObject.put("login", mLogin.getText().toString());
                            jsonObject.put("role", JsonConverter.convertToJSONObject(mRole.getSelectedItem()));
                            if (!mNaissance.getSelectedItem().equals(getString(R.string.modify_personne_hint_date)))
                                jsonObject.put("naissance", mNaissance.getSelectedItem().toString());
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        JsonObjectRequest jsonObjectRequest = new JsonObjectRequest(Request.Method.PUT, VolleyUtils.baseUri + "/personne/" + personne.getId(), jsonObject, new Response.Listener<JSONObject>() {
                            @Override
                            public void onResponse(JSONObject response) {
                                mProgressBar.setVisibility(View.GONE);
                                //Toast.makeText(getContext(), R.string.modify_done, Toast.LENGTH_LONG).show();
                                Toast.makeText(getContext(), "good response", Toast.LENGTH_SHORT).show();
                                dismiss();
                            }
                        }, new Response.ErrorListener() {
                            @Override
                            public void onErrorResponse(VolleyError error) {
                                mProgressBar.setVisibility(View.GONE);
                                if (error.networkResponse != null)
                                    Toast.makeText(getContext(), R.string.modify_refused + " : " + ConvertBytesToString.bytesToStringVolLey(error.networkResponse.data), Toast.LENGTH_LONG).show();
                                else
                                    Toast.makeText(getContext(), "!!" + error.getMessage(), Toast.LENGTH_SHORT).show();
                                mRole.setVisibility(View.VISIBLE);
                                mCarteInput.setVisibility(View.VISIBLE);
                                mLoginInput.setVisibility(View.VISIBLE);
                                mNaissance.setVisibility(View.VISIBLE);
                            }
                        }
                        ) {
                            @Override
                            public Map<String, String> getHeaders() throws AuthFailureError {
                                Map<String, String> headers = new HashMap<>();
                                headers.put("Authorization", MainActivity.session.getToken());
                                return headers;
                            }
                        };
                        jsonObjectRequest.setTag(ModifyPersonne.class);

                        try {
                            for (String s : jsonObjectRequest.getHeaders().keySet()) {
                                System.out.println(s + " " + jsonObjectRequest.getHeaders().get(s));
                            }
                        } catch (AuthFailureError authFailureError) {
                            authFailureError.printStackTrace();
                        }
                        System.out.println(ConvertBytesToString.bytesToStringVolLey(jsonObjectRequest.getBody()));

                        VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonObjectRequest);
                    }
                });
            }
        });
        mNaissance.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_UP) {
                    new DatePickerDialog(getContext(), date, myCalendar
                            .get(Calendar.YEAR), myCalendar.get(Calendar.MONTH),
                            myCalendar.get(Calendar.DAY_OF_MONTH)).show();
                }
                return true;
            }
        });
        dialog.setCanceledOnTouchOutside(false);
        return dialog;
    }

    private void updateLabel() {
        String myFormat = "yyyy-MM-dd"; //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, Locale.FRANCE);
        arraySpinnerNaissance[0] = sdf.format(myCalendar.getTime());
        adapterNaissance.notifyDataSetChanged();
    }


    @Override
    public void onStop() {
        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyPersonne.class);
        dismissAllowingStateLoss();
        super.onStop();
    }

    private void changeRolePossibility() {
        JsonArrayRequest jsonArrayRequest = new JsonArrayRequest(Request.Method.GET, VolleyUtils.baseUri + "/role", null, new Response.Listener<JSONArray>() {
            @Override
            public void onResponse(JSONArray response) {
                try {
                    arraySpinnerRole.clear();
                    for (int i = 0; i < response.length(); ++i) {
                        arraySpinnerRole.add(JsonConverter.getConverter().fromJson(String.valueOf(response.getJSONObject(i)), Role.class));
                    }
                    adapterRole.notifyDataSetChanged();
                } catch (JSONException e) {
                    e.printStackTrace();
                }

            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if (error.networkResponse == null)
                    Toast.makeText(getContext(), "Verifier votre connection Internet", Toast.LENGTH_LONG).show();
                else
                    Toast.makeText(getContext(), ConvertBytesToString.bytesToStringVolLey(error.networkResponse.data), Toast.LENGTH_LONG).show();
            }
        }
        );
        jsonArrayRequest.setTag(ModifyPersonne.class);
        VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonArrayRequest);
    }


    @Override
    public void processNFC(String idCard) {
        mCarte.setText(idCard);
    }
}
