package net.plil.clubinfo.etunicorn.app.personne;

import android.content.Context;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.PopupMenu;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import net.plil.clubinfo.etunicorn.R;
import net.plil.clubinfo.etunicorn.data.Personne;

import java.util.List;
import java.util.Locale;

class MyPersonneRecyclerViewAdapter extends RecyclerView.Adapter<MyPersonneRecyclerViewAdapter.ViewHolder> {

    private final List<Personne> mValues;
    private final PersonneFragment.OnListFragmentInteractionListener mListener;
    private final Context context;


    MyPersonneRecyclerViewAdapter(List<Personne> items, PersonneFragment.OnListFragmentInteractionListener listener, Context context) {
        mValues = items;
        mListener = listener;
        this.context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.fragment_personne, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        holder.mItem = mValues.get(position);
        holder.mLogin.setText(mValues.get(position).getLogin());
        holder.mCarte.setText(mValues.get(position).getCarte());
        holder.mRole.setText(mValues.get(position).getRole().getNom());
        holder.mSolde.setText(String.format(Locale.FRANCE, "%.2f", mValues.get(position).getSolde()));

        holder.mView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (null != mListener) {
                    mListener.onListFragmentInteraction(holder.mItem);
                }
            }
        });

        holder.mButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final PopupMenu popup = new PopupMenu(v.getContext(), holder.mButton);
                //inflating menu from xml resource
                popup.inflate(R.menu.option_list);
                //adding click listener
                popup.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
                    @Override
                    public boolean onMenuItemClick(MenuItem item) {
                        switch (item.getItemId()) {
                            case R.id.delete:
                                DeletePersonne deletePersonne = DeletePersonne.newInstance(holder.mItem);
                                deletePersonne.show(((AppCompatActivity) context).getSupportFragmentManager(), context.getString(R.string.tag_delete_personne));
                                break;
                            case R.id.modify:
                                ModifyPersonne modifyPersonne = ModifyPersonne.newInstance(holder.mItem);
                                modifyPersonne.show(((AppCompatActivity) context).getSupportFragmentManager(), context.getString(R.string.tag_modify_personne));
                                break;
                        }
                        return false;
                    }
                });
                //displaying the popup
                popup.show();
            }
        });

    }

    @Override
    public int getItemCount() {
        return mValues.size();
    }

     class ViewHolder extends RecyclerView.ViewHolder {
         final View mView;
         final TextView mLogin;
         final TextView mCarte;
         final TextView mRole;
         final TextView mSolde;
         final TextView mButton;
         Personne mItem;

         ViewHolder(View view) {
            super(view);
            mView = view;
            mLogin = (TextView) view.findViewById(R.id.personne_login);
            mCarte = (TextView) view.findViewById(R.id.personne_carte);
            mRole = (TextView) view.findViewById(R.id.personne_role);
            mSolde = (TextView) view.findViewById(R.id.personne_solde);
            mButton = (TextView) view.findViewById(R.id.personne_options);


        }
    }
}
