package net.plil.clubinfo.etunicorn.app.personne;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;

import net.plil.clubinfo.etunicorn.R;
import net.plil.clubinfo.etunicorn.app.MainActivity;
import net.plil.clubinfo.etunicorn.app.NFCSupport;
import net.plil.clubinfo.etunicorn.data.Personne;
import net.plil.clubinfo.etunicorn.utils.ConvertBytesToString;
import net.plil.clubinfo.etunicorn.utils.JsonConverter;
import net.plil.clubinfo.etunicorn.utils.VolleyUtils;

import org.json.JSONArray;
import org.json.JSONException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * A fragment representing a list of Items.
 * <p/>
 * Activities containing this fragment MUST implement the {@link OnListFragmentInteractionListener}
 * interface.
 */
public class PersonneFragment extends Fragment implements NFCSupport {

    private OnListFragmentInteractionListener mListener;
    private ArrayList<Personne> personnes = new ArrayList<>();
    private RecyclerView.Adapter mAdapter;
    private SwipeRefreshLayout mSwipeRefreshLayout;

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public PersonneFragment() {
    }

    public static PersonneFragment newInstance() {
        return new PersonneFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_personne_list, container, false);

        // Set the adapter
        Context context = view.getContext();
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.personne_list);
        recyclerView.setLayoutManager(new LinearLayoutManager(context));
        mAdapter = new MyPersonneRecyclerViewAdapter(personnes, mListener, getActivity());
        recyclerView.setAdapter(mAdapter);
        setPersonnes();
        FloatingActionButton fAB = (FloatingActionButton) view.findViewById(R.id.personne_add);
        fAB.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CreatePersonne newFragment = new CreatePersonne();
                newFragment.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        setPersonnes();
                    }
                });
                newFragment.show(getFragmentManager(), getString(R.string.tag_create_personne));
            }
        });

        mSwipeRefreshLayout = (SwipeRefreshLayout) view.findViewById(R.id.personne_swipeRefresh);
        mSwipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                setPersonnes();
            }
        });

        return view;
    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnListFragmentInteractionListener) {
            mListener = (OnListFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnListFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    private void setPersonnes() {
        JsonArrayRequest jsonArrayRequest = new JsonArrayRequest(Request.Method.GET, VolleyUtils.baseUri + "/personne", null, new Response.Listener<JSONArray>() {
            @Override
            public void onResponse(JSONArray response) {
                try {
                    personnes.clear();
                    for (int i = 0; i < response.length(); ++i) {
                        personnes.add(JsonConverter.getConverter().fromJson(String.valueOf(response.getJSONObject(i)), Personne.class));
                    }
                    mAdapter.notifyDataSetChanged();
                    mSwipeRefreshLayout.setRefreshing(false);
                } catch (JSONException e) {
                    e.printStackTrace();
                }

            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if (error.networkResponse == null)
                    Toast.makeText(getActivity(), "Unknow error SORRY", Toast.LENGTH_LONG).show();
                else
                    Toast.makeText(getActivity(), ConvertBytesToString.bytesToStringVolLey(error.networkResponse.data), Toast.LENGTH_LONG).show();
            }
        }
        ) {
            @Override
            public Map<String, String> getHeaders() throws AuthFailureError {
                Map<String, String> headers = new HashMap<>();
                headers.put("Authorization", MainActivity.session.getToken());
                return headers;
            }
        };
        jsonArrayRequest.setTag(ModifyPersonne.class);
        VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonArrayRequest);
    }

    @Override
    public void processNFC(String idCardUser) {
        CreatePersonne createPersonne = (CreatePersonne) getFragmentManager().findFragmentByTag(getString(R.string.tag_create_personne));
        if (createPersonne != null) {
            createPersonne.processNFC(idCardUser);
        } else {
            ModifyPersonne modifyPersonne = (ModifyPersonne) getFragmentManager().findFragmentByTag(getString(R.string.tag_modify_personne));
            if (modifyPersonne != null){
                modifyPersonne.processNFC(idCardUser);
            }
        }

    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p/>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnListFragmentInteractionListener {
        void onListFragmentInteraction(Personne personne);
    }
}
